import pytest
import xarray as xr
import pandas as pd
import numpy as np
from src.statistics import standardise, standardise_inverse, centre, centre_inverse


@pytest.fixture(scope='module')
def input_data():
    return np.array([np.random.normal(2, 2, 3000),
                     np.random.normal(-5, 3, 3000),
                     np.random.normal(10, 1, 3000)]).T


@pytest.fixture(scope='module')
def pandas(input_data):
    return pd.DataFrame(input_data)


@pytest.fixture(scope='module')
def xarray(input_data):
    return xr.DataArray(input_data, dims=['index', 'value'])


class TestStandardise:

    @pytest.mark.parametrize('data_org, dim', [(pytest.lazy_fixture('pandas'), 0),
                                               (pytest.lazy_fixture('xarray'), 'index')])
    def test_standardise(self, data_org, dim):
        mean, std, data = standardise(data_org, dim)
        assert np.testing.assert_almost_equal(mean, [2, -5, 10], decimal=1) is None
        assert np.testing.assert_almost_equal(std, [2, 3, 1], decimal=1) is None
        assert np.testing.assert_almost_equal(data.mean(dim), [0, 0, 0]) is None
        assert np.testing.assert_almost_equal(data.std(dim), [1, 1, 1]) is None

    @pytest.mark.parametrize('data_org, dim', [(pytest.lazy_fixture('pandas'), 0),
                                               (pytest.lazy_fixture('xarray'), 'index')])
    def test_standardise_inverse(self, data_org, dim):
        mean, std, data = standardise(data_org, dim)
        data_recovered = standardise_inverse(data, mean, std)
        assert np.testing.assert_array_almost_equal(data_org, data_recovered) is None


class TestCentre:

    @pytest.mark.parametrize('data_org, dim', [(pytest.lazy_fixture('pandas'), 0),
                                               (pytest.lazy_fixture('xarray'), 'index')])
    def test_centre(self, data_org, dim):
        mean, std, data = centre(data_org, dim)
        assert np.testing.assert_almost_equal(mean, [2, -5, 10], decimal=1) is None
        assert std is None
        assert np.testing.assert_almost_equal(data.mean(dim), [0, 0, 0]) is None

    @pytest.mark.parametrize('data_org, dim', [(pytest.lazy_fixture('pandas'), 0),
                                               (pytest.lazy_fixture('xarray'), 'index')])
    def test_centre_inverse(self, data_org, dim):
        mean, _, data = centre(data_org, dim)
        data_recovered = centre_inverse(data, mean)
        assert np.testing.assert_array_almost_equal(data_org, data_recovered) is None

